require_relative 'minitest_helper'

module Qdsl
  class SelectTest < QueryTest
    def test_single_table_select_single_column_string
      query = Qdsl::select('id').from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00', result.sql
    end

    def test_single_table_select_single_column_symbol
      query = Qdsl::select(:id).from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00', result.sql
    end

    def test_single_table_select_single_column_method
      query = Qdsl::select { |t| t.id }.from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00', result.sql
    end

    def test_single_table_select_single_column_name_string
      query = Qdsl::select { |t| t['id'] }.from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00', result.sql
    end

    def test_single_table_select_single_column_name_symbol
      query = Qdsl::select { |t| t[:id] }.from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00', result.sql
    end

    def test_single_table_select_two_columns_string
      query = Qdsl::select('id', 'first_name').from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id, _00.first_name FROM clients AS _00', result.sql
    end

    def test_single_table_select_two_columns_symbol
      query = Qdsl::select(:id, :first_name).from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id, _00.first_name FROM clients AS _00', result.sql
    end

    def test_single_table_select_two_columns_method
      query = Qdsl::select { |t| [t.id, t.first_name] }.from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id, _00.first_name FROM clients AS _00', result.sql
    end

    def test_single_table_select_two_columns_name_string
      query = Qdsl::select { |t| [t['id'], t['first_name']] }.from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id, _00.first_name FROM clients AS _00', result.sql
    end

    def test_single_table_select_two_columns_name_symbol
      query = Qdsl::select { |t| [t[:id], t[:first_name]] }.from(@clients)
      result = query.render
      assert_sql 'SELECT _00.id, _00.first_name FROM clients AS _00', result.sql
    end

    def test_two_tables_select_single_column_string
      query = Qdsl::select('id').from(@clients).inner_join(@members).on { |t0, t1| t0.id.equals(t1.id) }
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00 INNER JOIN members AS _01 ON _00.id = _01.id', result.sql
    end

    def test_two_tables_select_single_column_symbol
      query = Qdsl::select(:id).from(@clients).inner_join(@members).on { |t0, t1| t0.id.equals(t1.id) }
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00 INNER JOIN members AS _01 ON _00.id = _01.id', result.sql
    end

    def test_two_tables_select_single_column_method
      query = Qdsl::select { |t0, t1| t0.id }.from(@clients).inner_join(@members).on { |t0, t1| t0.id.equals(t1.id) }
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00 INNER JOIN members AS _01 ON _00.id = _01.id', result.sql
    end

    def test_two_tables_select_single_column_name_string
      query = Qdsl::select { |t0, t1| t0['id'] }.from(@clients).inner_join(@members).on { |t0, t1| t0.id.equals(t1.id) }
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00 INNER JOIN members AS _01 ON _00.id = _01.id', result.sql
    end

    def test_two_tables_select_single_column_name_symbol
      query = Qdsl::select { |t0, t1| t0[:id] }.from(@clients).inner_join(@members).on { |t0, t1| t0.id.equals(t1.id) }
      result = query.render
      assert_sql 'SELECT _00.id FROM clients AS _00 INNER JOIN members AS _01 ON _00.id = _01.id', result.sql
    end

    def test_limit
      query = Qdsl::select(:id).from(@clients).limit(5)
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT _00.id
FROM clients AS _00
LIMIT 5
      SQL
    end

    def test_order_by_string
      query = Qdsl::select(:id).from(@clients).limit(5).order_by('first_name')
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT _00.id
FROM clients AS _00
ORDER BY _00.first_name
LIMIT 5
      SQL
    end

    def test_order_by_symbol
      query = Qdsl::select(:id).from(@clients).limit(5).order_by(:first_name)
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT _00.id
FROM clients AS _00
ORDER BY _00.first_name
LIMIT 5
      SQL
    end

    def test_order_by_method
      query = Qdsl::select(:id).from(@clients).limit(5).order_by { |t| t.first_name }
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.id
FROM clients AS _00
ORDER BY _00.first_name
LIMIT 5
      SQL
    end

    def test_call
      query = Qdsl::select { |t| [t.first_name, t._call('AGE', t.birthdate).as(:age)] }.
        from(@clients)
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.first_name,
  AGE(_00.birthdate) AS age
FROM clients AS _00
      SQL
    end

    def test_formatcall
      query = Qdsl::select { |t| [t.first_name, t._formatcall('EXTRACT(year FROM %1)', t._call('AGE', t.birthdate)).as(:age)] }.
        from(@clients)
      result = query.render

      assert_sql <<-SQL, result.sql
SELECT
  _00.first_name,
  EXTRACT(year FROM AGE(_00.birthdate)) AS age
FROM clients AS _00
      SQL
    end
  end
end

