require_relative 'minitest_helper'

module Qdsl
  class SubqueryTest < QueryTest
    def test_simple
      subquery = Qdsl::select(:id, :first_name, :last_name).from(@clients)
      query = Qdsl::select(:first_name).from(subquery)

      result = query.render

      assert_nil result.id
      assert_equal ['first_name'], result.column_names
      assert_sql <<-SQL, result.sql
SELECT
  _00.first_name
FROM (
  SELECT
    _01.id,
    _01.first_name,
    _01.last_name
  FROM clients AS _01
) AS _00
      SQL
      assert_equal({}, result.parameters)
    end

    def test_inner_join
      subquery = Qdsl::select(:id, :first_name, :last_name).from(@clients)
      query = Qdsl::select { |t0, t1| [t0.first_name, t1.last_name] }.
        from(subquery).
        inner_join(subquery).on { |t0, t1| t0.id.equals(t1.id) }

      result = query.render

      assert_nil result.id
      assert_equal ['first_name', 'last_name'], result.column_names
      assert_sql <<-SQL, result.sql
SELECT
  _00.first_name,
  _02.last_name
FROM (
  SELECT
    _01.id,
    _01.first_name,
    _01.last_name
  FROM clients AS _01
) AS _00
INNER JOIN (
  SELECT
    _03.id,
    _03.first_name,
    _03.last_name
  FROM clients AS _03
) AS _02
ON _00.id = _02.id
      SQL
      assert_equal({}, result.parameters)
    end

    def test_inner_join_where
      subquery = Qdsl::select(:id, :first_name, :last_name).from(@clients)
      query = Qdsl::select { |t0, t1| [t0.first_name, t1.last_name] }.
        from(subquery).
        inner_join(subquery).on { |t0, t1| t0.id.equals(t1.id) }.
        where { |t0, t1| t0.first_name.equals('str0').and(t1.last_name.equals('str1')) }

      result = query.render

      assert_nil result.id
      assert_equal ['first_name', 'last_name'], result.column_names
      assert_sql <<-SQL, result.sql
SELECT
  _00.first_name,
  _02.last_name
FROM (
  SELECT
    _01.id,
    _01.first_name,
    _01.last_name
  FROM clients AS _01
) AS _00
INNER JOIN (
  SELECT
    _03.id,
    _03.first_name,
    _03.last_name
  FROM clients AS _03
) AS _02
ON _00.id = _02.id
WHERE (_00.first_name = :_param00) AND (_02.last_name = :_param01)
        SQL
      assert_equal({'_param00' => 'str0', '_param01' => 'str1'}, result.parameters)
    end

    def test_computed_column
      subquery = Qdsl::select(:id) do |t| [
        t.first_name,
        t.last_name,
        age_in_years(t, :birthdate, :age)
      ] end.from(@clients)

      query = Qdsl::select(:first_name, :last_name, :age).from(subquery)

      result = query.render

      assert_nil result.id
      assert_equal ['first_name', 'last_name', 'age'], result.column_names
      assert_equal <<-SQL, result.sql
SELECT
  _00.first_name,
  _00.last_name,
  _00.age
FROM (
  SELECT
    _01.id,
    _01.first_name,
    _01.last_name,
    EXTRACT(year FROM AGE(_01.birthdate)) AS age
  FROM clients AS _01
) AS _00
  SQL
    end

    def test_computed_column_inner_join
      subquery = Qdsl::select(:id) do |t| [
        t.first_name,
        t.last_name,
        age_in_years(t, :birthdate, :age)
      ] end.from(@clients)

      query = Qdsl::select(:first_name, :last_name, :age).from(subquery).
        inner_join(subquery).on { |t0, t1| t0.id.equals(t1.id) }

      result = query.render

      assert_nil result.id
      assert_equal ['first_name', 'last_name', 'age'], result.column_names
      assert_equal <<-SQL, result.sql
SELECT
  _00.first_name,
  _00.last_name,
  _00.age
FROM (
  SELECT
    _01.id,
    _01.first_name,
    _01.last_name,
    EXTRACT(year FROM AGE(_01.birthdate)) AS age
  FROM clients AS _01
) AS _00
INNER JOIN (
  SELECT
    _03.id,
    _03.first_name,
    _03.last_name,
    EXTRACT(year FROM AGE(_03.birthdate)) AS age
  FROM clients AS _03
) AS _02
ON _00.id = _02.id
      SQL
    end

    def test_computed_column_complex_select
      subquery = Qdsl::select(:id) do |t| [
        t.first_name,
        t.last_name,
        age_in_years(t, :birthdate, :age)
      ] end.from(@clients)

      query = Qdsl::select(:first_name, :last_name) do |t0, t1|
        [t0.age, t1.first_name]
      end.from(subquery).
        inner_join(subquery).on { |t0, t1| t0.id.equals(t1.id) }

      result = query.render

      assert_nil result.id
      assert_equal ['first_name', 'last_name', 'age', 'first_name'], result.column_names
      assert_equal <<-SQL, result.sql
SELECT
  _00.first_name,
  _00.last_name,
  _00.age,
  _02.first_name
FROM (
  SELECT
    _01.id,
    _01.first_name,
    _01.last_name,
    EXTRACT(year FROM AGE(_01.birthdate)) AS age
  FROM clients AS _01
) AS _00
INNER JOIN (
  SELECT
    _03.id,
    _03.first_name,
    _03.last_name,
    EXTRACT(year FROM AGE(_03.birthdate)) AS age
  FROM clients AS _03
) AS _02
ON _00.id = _02.id
      SQL
    end

    private

    def age_in_years(t, column_name, name)
      t._formatcall(
        'EXTRACT(year FROM %1)',
        t._call('AGE', t[column_name])).as(name)
    end
  end
end

